# -*- coding: utf-8 -*-

from b3j0f.utils.ut import UTCase
from mock import MagicMock
from unittest import main

from link.kvstore.driver import Driver


class DriverTest(UTCase):
    def setUp(self):
        self.conn = 'myconn'

        self.driver = Driver()
        self.driver._connect = MagicMock(return_value=self.conn)
        self.driver._disconnect = MagicMock()
        self.driver._isconnected = MagicMock(
            side_effect=lambda conn: conn is not None
        )

    def tearDown(self):
        self.driver.disconnect()

        self.driver._connect.assert_called_once_with()
        self.driver._disconnect.assert_called_once_with(self.conn)
        self.driver._isconnected.assert_any_call(self.conn)

    def test_get_item(self):
        self.driver._get = MagicMock(return_value='bar')

        result = self.driver.get('foo')

        self.assertEqual(result, 'bar')
        self.driver._get.assert_called_with(self.conn, 'foo')

    def test_get_noitem(self):
        self.driver._get = MagicMock(side_effect=KeyError)

        with self.assertRaises(KeyError):
            self.driver.get('foo')

        self.driver._get.assert_called_with(self.conn, 'foo')

    def test_put_item(self):
        self.driver._put = MagicMock()

        self.driver.put('foo', 'bar')

        self.driver._put.assert_called_with(self.conn, 'foo', 'bar')

    def test_del_item(self):
        self.driver._remove = MagicMock()

        self.driver.remove('foo')

        self.driver._remove.assert_called_with(self.conn, 'foo')

    def test_contains_item(self):
        self.driver._exists = MagicMock(return_value=True)

        result = self.driver.exists('foo')

        self.assertTrue(result)
        self.driver._exists.assert_called_with(self.conn, 'foo')

    def test_get_keys(self):
        expected = ['foo', 'bar', 'baz']
        self.driver._keys = MagicMock(return_value=expected)

        result = self.driver.keys()

        self.assertEqual(result, expected)
        self.driver._keys.assert_called_with(self.conn)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from link.middleware.core import Middleware
from link.kvstore.driver import Driver


class KeyValueStore(Middleware):
    """
    Key/Value store middleware providing dict API.

    :param backend: Store driver to use
    :type backend: Driver
    """

    __constraints__ = [Driver]
    __protocols__ = ['kvstore']

    def __init__(self, backend, *args, **kwargs):
        super(KeyValueStore, self).__init__(*args, **kwargs)

        self._backend = backend

    def __getitem__(self, attr):
        return self._backend.get(attr)

    def __setitem__(self, attr, val):
        self._backend.put(attr, val)

    def __delitem__(self, attr):
        self._backend.remove(attr)

    def __contains__(self, attr):
        return self._backend.exists(attr)

    def __iter__(self):
        return iter(self._backend.keys())



# -*- coding: utf-8 -*-

from link.middleware.connectable import ConnectableMiddleware


class Driver(ConnectableMiddleware):
    """
    Generic driver for accessing Key/Value store.
    """

    def _get(self, conn, key):
        raise NotImplementedError()

    def _put(self, conn, key, val):
        raise NotImplementedError()

    def _remove(self, conn, key):
        raise NotImplementedError()

    def _exists(self, conn, key):
        raise NotImplementedError()

    def _keys(self, conn):
        raise NotImplementedError()

    def get(self, key):
        """
        Get value from the store.

        :param key: Associated key
        :type key: str

        :returns: Value

        :raises KeyError: if key does not exist
        """

        return self._get(self.conn, key)

    def put(self, key, val):
        """
        Set value in the store.

        :param key: Associated key
        :type key: str

        :param val: Value to set
        :type val: any
        """

        self._put(self.conn, key, val)

    def remove(self, key):
        """
        Remove value from the store.

        :param key: Associated key
        :type key: str

        :raises KeyError: if key does not exist
        """

        self._remove(self.conn, key)

    def exists(self, key):
        """
        Check if key exists in store.

        :param key: Key to check for
        :type key: str

        :rtype: bool
        """

        return self._exists(self.conn, key)

    def keys(self):
        """
        Get list of keys in store.

        :returns: List of keys
        :rtype: iterable
        """

        return self._keys(self.conn)



# -*- coding: utf-8 -*-

from b3j0f.utils.ut import UTCase
from unittest import main
from mock import Mock

from link.kvstore.core import KeyValueStore
from link.kvstore.driver import Driver


class KVStoreTest(UTCase):
    def setUp(self):
        self.backend = Mock()
        self.backend.__class__ = Driver

        self.store = KeyValueStore(self.backend)

    def test_get_item(self):
        attrs = {
            'get.return_value': 'bar'
        }
        self.backend.configure_mock(**attrs)

        result = self.store['foo']

        self.assertEqual(result, 'bar')
        self.backend.get.assert_called_with('foo')

    def test_get_noitem(self):
        attrs = {
            'get.side_effect': KeyError
        }
        self.backend.configure_mock(**attrs)

        with self.assertRaises(KeyError):
            self.store['foo']

        self.backend.get.assert_called_with('foo')

    def test_set_item(self):
        attrs = {
            'put.return_value': None
        }
        self.backend.configure_mock(**attrs)

        self.store['foo'] = 'bar'

        self.backend.put.assert_called_with('foo', 'bar')

    def test_del_item(self):
        attrs = {
            'remove.return_value': None
        }
        self.backend.configure_mock(**attrs)

        del self.store['foo']

        self.backend.remove.assert_called_with('foo')

    def test_del_noitem(self):
        attrs = {
            'remove.side_effect': KeyError
        }
        self.backend.configure_mock(**attrs)

        with self.assertRaises(KeyError):
            del self.store['foo']

        self.backend.remove.assert_called_with('foo')

    def test_contains_item(self):
        attrs = {
            'exists.return_value': True
        }
        self.backend.configure_mock(**attrs)

        result = 'foo' in self.store

        self.assertTrue(result)
        self.backend.exists.assert_called_with('foo')

    def test_iter_keys(self):
        expected = ['foo', 'bar', 'baz']
        attrs = {
            'keys.return_value': expected
        }
        self.backend.configure_mock(**attrs)

        result = [key for key in self.store]

        self.assertEqual(result, expected)
        self.backend.keys.assert_called_with()


if __name__ == '__main__':
    main()
